@@ -48,13 +48,14 @@ add_mime_cb (GtkButton *button, GladeXML *dialog)
 {
 	GtkTreeView     *treeview;
 	GtkTreeModel    *model;
-	GObject         *add_dialog;
+	GObject		*add_dialog;
 
 	treeview = GTK_TREE_VIEW (WID ("mime_types_tree"));
 	model = gtk_tree_view_get_model (treeview);
 
 	add_dialog = mime_add_dialog_new (model,
-		GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (button))));
+		GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (button))),
+		NULL);
 }
 
 static void
@@ -62,12 +63,11 @@ add_service_cb (GtkButton *button, GladeXML *dialog)
 {
 	GtkTreeView     *treeview;
 	GtkTreeModel    *model;
-	GObject         *add_dialog;
 
 	treeview = GTK_TREE_VIEW (WID ("mime_types_tree"));
 	model = gtk_tree_view_get_model (treeview);
 
-	add_dialog = service_add_dialog_new (model);
+	service_add_dialog_new (model);
 }
 
 static GObject *
@@ -84,8 +84,10 @@ launch_edit_dialog (GtkTreeModel *model, GtkTreeIter *iter)
 	case MODEL_ENTRY_SERVICE:
 		return service_edit_dialog_new (model, SERVICE_INFO (entry));
 
+#if 0
 	case MODEL_ENTRY_CATEGORY:
 		return mime_category_edit_dialog_new (model, MIME_CATEGORY_INFO (entry));
+#endif
 
 	default:
 		return NULL;
@@ -119,14 +121,7 @@ row_activated_cb (GtkTreeView *view, GtkTreePath *path, GtkTreeViewColumn *colum
 }
 
 static void
-edit_count_cb (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gint *count) 
-{
-	if (MODEL_ENTRY_FROM_ITER (iter)->type != MODEL_ENTRY_SERVICES_CATEGORY)
-		(*count)++;
-}
-
-static void
-remove_count_cb (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gint *count) 
+selection_count_cb (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gint *count) 
 {
 	if (MODEL_ENTRY_FROM_ITER (iter)->type != MODEL_ENTRY_SERVICES_CATEGORY &&
 	    MODEL_ENTRY_FROM_ITER (iter)->type != MODEL_ENTRY_CATEGORY)
@@ -138,21 +133,10 @@ selection_changed_cb (GtkTreeSelection *selection, GladeXML *dialog)
 {
 	gint count = 0;
 
-	gtk_tree_selection_selected_foreach (selection, (GtkTreeSelectionForeachFunc) edit_count_cb, &count);
+	gtk_tree_selection_selected_foreach (selection, (GtkTreeSelectionForeachFunc) selection_count_cb, &count);
 
-	if (count == 0)
-		gtk_widget_set_sensitive (WID ("edit_button"), FALSE);
-	else
-		gtk_widget_set_sensitive (WID ("edit_button"), TRUE);
-
-	count = 0;
-
-	gtk_tree_selection_selected_foreach (selection, (GtkTreeSelectionForeachFunc) remove_count_cb, &count);
-
-	if (count == 0)
-		gtk_widget_set_sensitive (WID ("remove_button"), FALSE);
-	else
-		gtk_widget_set_sensitive (WID ("remove_button"), TRUE);
+	gtk_widget_set_sensitive (WID ("edit_button"), count != 0);
+	gtk_widget_set_sensitive (WID ("remove_button"), count != 0);
 }
 
 static void
@@ -208,28 +192,37 @@ create_dialog (void)
 
 	/* Icon/description column */
 	column = gtk_tree_view_column_new ();
+	gtk_tree_view_column_set_resizable (column, TRUE);
+	gtk_tree_view_column_set_title (column, _("Description"));
 
 	renderer = gtk_cell_renderer_pixbuf_new ();
 	gtk_tree_view_column_pack_start (column, renderer, FALSE);
-	gtk_tree_view_column_set_attributes
-		(column, renderer, "pixbuf", MODEL_COLUMN_ICON, NULL);
+	gtk_tree_view_column_set_attributes 
+		(column, renderer, "pixbuf", MODEL_COLUMN_ICON, NULL); 
 
 	/* Description column */
 	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_column_pack_start (column, renderer, FALSE);
 	gtk_tree_view_column_set_attributes
 		(column, renderer, "text", MODEL_COLUMN_DESCRIPTION, NULL);
+	g_object_set (G_OBJECT (treeview), 
+		"search_column", MODEL_COLUMN_DESCRIPTION,
+		NULL);
 
-	gtk_tree_view_column_set_title (column, _("Description"));
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 	gtk_tree_view_set_expander_column (GTK_TREE_VIEW (treeview), column);
 
 	/* Extensions column */
+	column = gtk_tree_view_column_new ();
+	gtk_tree_view_column_set_resizable (column, TRUE);
+	gtk_tree_view_column_set_title (column, _("Extensions"));
+
 	renderer = gtk_cell_renderer_text_new ();
-	gtk_tree_view_insert_column_with_attributes
-		(GTK_TREE_VIEW (treeview), -1, _("Extensions"), renderer,
-		 "text", MODEL_COLUMN_FILE_EXT,
-		 NULL);
+	gtk_tree_view_column_pack_start (column, renderer, TRUE);
+	gtk_tree_view_column_set_attributes 
+		(column, renderer, "text", MODEL_COLUMN_FILE_EXT, NULL);
+
+	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
 	gtk_widget_set_sensitive (WID ("edit_button"), FALSE);
 	gtk_widget_set_sensitive (WID ("remove_button"), FALSE);
@@ -255,9 +248,6 @@ create_dialog (void)
 static void
 dialog_done_cb (MimeEditDialog *dialog, gboolean done, MimeTypeInfo *info) 
 {
-	if (done)
-		mime_type_info_save (info);
-
 	gtk_main_quit ();
 }
 
@@ -265,33 +255,52 @@ dialog_done_cb (MimeEditDialog *dialog, gboolean done, MimeTypeInfo *info)
 int
 main (int argc, char **argv) 
 {
-	gchar        *mime_type;
+	char const   *mime_type = NULL;
+	char const   *file_name = NULL;
 	GtkTreeModel *model;
-	MimeTypeInfo *info = NULL;
-	GObject      *mime_dialog;
+	GnomeProgram *program;
+	poptContext   popt_ctxt = 0;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	if (argc >= 1)
-		mime_type = g_strdup (argv[1]);
-	else
-		mime_type = NULL;
+	program = gnome_program_init ("gnome-file-types-properties",
+		VERSION, LIBGNOMEUI_MODULE,
+		argc, argv,
+		NULL);
+	g_object_get (G_OBJECT (program),
+		GNOME_PARAM_POPT_CONTEXT, &popt_ctxt,
+		NULL);
 
-	gnome_program_init ("gnome-file-types-properties", VERSION, LIBGNOMEUI_MODULE, argc, argv, NULL);
+	if (popt_ctxt) {
+		const gchar **startup_files = poptGetArgs (popt_ctxt);
+		if (startup_files != NULL) {
+			mime_type = startup_files [0];
+			if (mime_type != NULL)
+				file_name = startup_files [1];
+		}
+	}
 
 	if (mime_type == NULL) {
-		GladeXML *dialog = create_dialog ();
-		gtk_widget_show_all (WID ("main_dialog"));
+		GladeXML  *dialog = create_dialog ();
+		GtkWidget *mime_dialog = WID ("main_dialog");
+		gtk_dialog_set_default_response (GTK_DIALOG (mime_dialog),
+			GTK_RESPONSE_CLOSE);
+		capplet_set_icon (mime_dialog, "gnome-ccmime.png");
+		gtk_widget_show_all (mime_dialog);
 	} else {
+		MimeTypeInfo *info = NULL;
+		GObject *mime_dialog;
+
 		model = GTK_TREE_MODEL (mime_types_model_new (FALSE));
 		if (strcmp (GNOME_VFS_MIME_TYPE_UNKNOWN, mime_type)) {
 			info = mime_type_info_new (mime_type, model);
 			mime_dialog = mime_edit_dialog_new (model, info);
 		} else
-			mime_dialog = mime_add_dialog_new (model, NULL);
-		g_signal_connect (mime_dialog, "done", (GCallback) dialog_done_cb, info);
+			mime_dialog = mime_add_dialog_new (model, NULL, file_name);
+		g_signal_connect (mime_dialog,
+			"done", (GCallback) dialog_done_cb, info);
 	}
 
 	gtk_main ();
